@@ -35,7 +35,7 @@ await_ide(u8 mask, u8 flags, u16 base, u16 timeout)
         u8 status = inb(base+ATA_CB_STAT);
         if ((status & mask) == flags)
             return status;
-        if (check_time(end)) {
+        if (check_tsc(end)) {
             warn_timeout();
             return -1;
         }
@@ -106,7 +106,7 @@ ata_reset(struct atadrive_s *adrive_g)
             if (inb(iobase1 + ATA_CB_DH) == ATA_CB_DH_DEV1)
                 break;
             // Change drive request failed to take effect - retry.
-            if (check_time(end)) {
+            if (check_tsc(end)) {
                 warn_timeout();
                 goto done;
             }
@@ -405,8 +405,6 @@ ata_try_dma(struct disk_op_s *op, int iswrite, int blocksize)
             // Too many descriptors..
             return -1;
         u32 count = bytes;
-        if (count > 0x10000)
-            count = 0x10000;
         u32 max = 0x10000 - (dest & 0xffff);
         if (count > max)
             count = max;
@@ -455,7 +453,7 @@ ata_dma_transfer(struct disk_op_s *op)
         if (status & BM_STATUS_IRQ)
             break;
         // Transfer in progress
-        if (check_time(end)) {
+        if (check_tsc(end)) {
             // Timeout.
             warn_timeout();
             break;
@@ -664,6 +662,8 @@ fail:
 int
 process_atapi_op(struct disk_op_s *op)
 {
+    if (!CONFIG_ATA)
+        return 0;
     switch (op->command) {
     case CMD_READ:
         return cdb_read(op);
@@ -774,7 +774,7 @@ init_drive_atapi(struct atadrive_s *dummy, u16 *buffer)
     snprintf(adrive_g->drive.desc, MAXDESCSIZE, "ata%d-%d: %s ATAPI-%d %s"
              , adrive_g->chan_gf->chanid, adrive_g->slave
              , extract_model(model, buffer), extract_version(buffer)
-             , (iscd ? "CD-Rom/DVD-Rom" : "Device"));
+             , (iscd ? "DVD/CD" : "Device"));
     dprintf(1, "%s\n", adrive_g->drive.desc);
 
     // fill cdidmap
@@ -850,7 +850,7 @@ powerup_await_non_bsy(u16 base)
             dprintf(4, "powerup IDE floating\n");
             return orstatus;
         }
-        if (check_time(SpinupEnd)) {
+        if (check_tsc(SpinupEnd)) {
             warn_timeout();
             return -1;
         }
@@ -986,8 +986,10 @@ ata_init(void)
 
         u32 port1, port2, irq;
         if (prog_if & 1) {
-            port1 = pci_config_readl(bdf, PCI_BASE_ADDRESS_0) & ~3;
-            port2 = pci_config_readl(bdf, PCI_BASE_ADDRESS_1) & ~3;
+            port1 = (pci_config_readl(bdf, PCI_BASE_ADDRESS_0)
+                     & PCI_BASE_ADDRESS_IO_MASK);
+            port2 = (pci_config_readl(bdf, PCI_BASE_ADDRESS_1)
+                     & PCI_BASE_ADDRESS_IO_MASK);
             irq = pciirq;
         } else {
             port1 = PORT_ATA1_CMD_BASE;
@@ -998,8 +1000,10 @@ ata_init(void)
         count++;
 
         if (prog_if & 4) {
-            port1 = pci_config_readl(bdf, PCI_BASE_ADDRESS_2) & ~3;
-            port2 = pci_config_readl(bdf, PCI_BASE_ADDRESS_3) & ~3;
+            port1 = (pci_config_readl(bdf, PCI_BASE_ADDRESS_2)
+                     & PCI_BASE_ADDRESS_IO_MASK);
+            port2 = (pci_config_readl(bdf, PCI_BASE_ADDRESS_3)
+                     & PCI_BASE_ADDRESS_IO_MASK);
             irq = pciirq;
         } else {
             port1 = PORT_ATA2_CMD_BASE;
@@ -1034,5 +1038,5 @@ ata_setup(void)
 
     SET_BDA(disk_control_byte, 0xc0);
 
-    enable_hwirq(14, entry_76);
+    enable_hwirq(14, FUNC16(entry_76));
 }
