@@ -57,35 +57,6 @@ __call16_int(struct bregs *callregs, u16 offset)
     call16(callregs);
 }
 
-// 16bit trampoline for enabling irqs from 32bit mode.
-ASM16(
-    "  .global trampoline_checkirqs\n"
-    "trampoline_checkirqs:\n"
-    "  rep ; nop\n"
-    "  lretw"
-    );
-
-void
-check_irqs(void)
-{
-    if (MODE16) {
-        asm volatile(
-            "sti\n"
-            "nop\n"
-            "rep ; nop\n"
-            "cli\n"
-            "cld\n"
-            : : :"memory");
-    } else {
-        extern void trampoline_checkirqs();
-        struct bregs br;
-        br.flags = F_IF;
-        br.code.seg = SEG_BIOS;
-        br.code.offset = (u32)&trampoline_checkirqs;
-        call16big(&br);
-    }
-}
-
 
 /****************************************************************
  * String ops
@@ -194,6 +165,17 @@ memcpy_far(u16 d_seg, void *d_far, u16 s_seg, const void *s_far, size_t len)
         : "cc", "memory");
 }
 
+inline void
+memcpy_fl(void *d_fl, const void *s_fl, size_t len)
+{
+    if (MODESEGMENT)
+        memcpy_far(FLATPTR_TO_SEG(d_fl), (void*)FLATPTR_TO_OFFSET(d_fl)
+                   , FLATPTR_TO_SEG(s_fl), (void*)FLATPTR_TO_OFFSET(s_fl)
+                   , len);
+    else
+        memcpy(d_fl, s_fl, len);
+}
+
 void *
 #undef memcpy
 memcpy(void *d1, const void *s1, size_t len)
@@ -220,24 +202,27 @@ memcpy(void *d1, const void *s1, size_t len)
     return d1;
 }
 
-// Copy from memory mapped IO.  IO mem is very slow, so yield
-// periodically.  'len' must be 4 byte aligned.
+// Copy to/from memory mapped IO.  IO mem is very slow, so yield
+// periodically.
 void
 iomemcpy(void *d, const void *s, u32 len)
 {
     yield();
-    while (len) {
+    while (len > 3) {
         u32 copylen = len;
         if (copylen > 2048)
             copylen = 2048;
-        len -= copylen;
         copylen /= 4;
+        len -= copylen * 4;
         asm volatile(
             "rep movsl (%%esi),%%es:(%%edi)"
             : "+c"(copylen), "+S"(s), "+D"(d)
             : : "cc", "memory");
         yield();
     }
+    if (len)
+        // Copy any remaining bytes.
+        memcpy(d, s, len);
 }
 
 void *
@@ -273,19 +258,6 @@ strtcpy(char *dest, const char *src, size_t len)
  * Keyboard calls
  ****************************************************************/
 
-// Wait for 'usec' microseconds using (with irqs enabled) using int 1586.
-void
-biosusleep(u32 usec)
-{
-    struct bregs br;
-    memset(&br, 0, sizeof(br));
-    br.flags = F_IF;
-    br.ah = 0x86;
-    br.cx = usec >> 16;
-    br.dx = usec;
-    call16_int(0x15, &br);
-}
-
 // See if a keystroke is pending in the keyboard buffer.
 static int
 check_for_keystroke(void)
@@ -313,12 +285,12 @@ get_raw_keystroke(void)
 int
 get_keystroke(int msec)
 {
+    u32 end = calc_future_timer(msec);
     for (;;) {
         if (check_for_keystroke())
             return get_raw_keystroke();
-        if (msec <= 0)
+        if (check_timer(end))
             return -1;
-        biosusleep(50*1000);
-        msec -= 50;
+        wait_irq();
     }
 }
